@@ -4,7 +4,8 @@ import logging
 from urllib.parse import urljoin
 from base64 import urlsafe_b64decode
 import requests
-from flask import Request, abort
+from flask import Request, abort, jsonify
+from werkzeug.exceptions import Forbidden, BadRequest
 from . import assets
 
 GCP_PROJECT = os.environ["GCP_PROJECT"]
@@ -20,6 +21,47 @@ class ServiceError(Exception):
         super().__init__(message)
 
 
+def _authorize(request, asset, permission):
+    uid = user_info(request)["user_id"]
+    resp = call("check_permission", uid=uid, asset=asset, permission=permission)
+    if not resp["granted"]:
+        resp = jsonify(message="Access denied")
+        resp.status_code = 403
+        abort(resp)
+
+
+def authorize(permission):
+    def wrap(func):
+        def wrapped_func(request: Request):
+            asset_type = permission.split(".")[0]
+            try:
+                asset = getattr(assets, asset_type)(**request.json)
+            except KeyError as e:
+                resp = jsonify(message=f"Missing parameter: {e}")
+                resp.status_code = 400
+                abort(resp)
+            _authorize(request, asset, permission)
+            request.headers["X-Authorized-Asset"] = asset
+            return func(request)
+
+        return wrapped_func
+
+    return wrap
+
+
+def authorized_asset(request: Request):
+    return request.headers.get("X-Authorized-Asset")
+
+
+def user_info(request):
+    encoded_user_info = request.headers.get("X-Endpoint-Api-Userinfo")
+    if encoded_user_info:
+        if not encoded_user_info.endswith("=="):
+            encoded_user_info += "=="
+        return json.loads(urlsafe_b64decode(encoded_user_info))
+    return None
+
+
 def get_auth_header(audience):
     if os.getenv("LOCAL"):
         from subprocess import Popen, PIPE
@@ -52,40 +94,3 @@ def call(name, **kwargs):
     logging.info(resp)
     r.raise_for_status()
     return resp
-
-
-def user_info(request):
-    encoded_user_info = request.headers.get("X-Endpoint-Api-Userinfo")
-    if encoded_user_info:
-        if not encoded_user_info.endswith("=="):
-            encoded_user_info += "=="
-        return json.loads(urlsafe_b64decode(encoded_user_info))
-    return None
-
-
-def _authorize(request, asset, permission):
-    uid = user_info(request)["user_id"]
-    granted = call(
-        "check_permission",
-        uid=uid,
-        asset=asset,
-        permission=permission,
-    )["granted"]
-    if not granted:
-        abort(403)
-
-
-def authorize(permission):
-    def wrap(func):
-        def wrapped_func(request: Request):
-            asset_type = permission.split(".")[0]
-            asset = getattr(assets, asset_type)(**request.json)
-            _authorize(request, asset, permission)
-            request.headers["X-Authorized-Asset"] = asset
-            return func(request)
-        return wrapped_func
-    return wrap
-
-
-def authorized_asset(request: Request):
-    return request.headers.get("X-Authorized-Asset")
